@@ -63,19 +63,10 @@ tracked_widget_free (gpointer data)
   TrackedWidget *tracked = data;
 
   if (tracked->widget != NULL)
-    {
-      dzl_clear_signal_handler (tracked->widget, &tracked->handler);
-      tracked->widget = NULL;
-    }
-
-  tracked->handler = 0;
-  tracked->id = 0;
-
+    g_signal_handler_disconnect (tracked->widget, tracked->handler);
   g_slice_free (TrackedWidget, tracked);
 }
 
-G_DEFINE_AUTOPTR_CLEANUP_FUNC (TrackedWidget, tracked_widget_free)
-
 static void
 dzl_preferences_view_track (DzlPreferencesView *self,
                             guint               id,
@@ -858,10 +849,10 @@ dzl_preferences_view_remove_id (DzlPreferences *preferences,
 {
   DzlPreferencesView *self = (DzlPreferencesView *)preferences;
   DzlPreferencesViewPrivate *priv = dzl_preferences_view_get_instance_private (self);
-  g_autoptr(TrackedWidget) tracked = NULL;
+  TrackedWidget *tracked;
 
   g_assert (DZL_IS_PREFERENCES_VIEW (self));
-  g_assert (widget_id != 0);
+  g_assert (widget_id);
 
   tracked = g_hash_table_lookup (priv->widgets, GUINT_TO_POINTER (widget_id));
 
@@ -869,10 +860,7 @@ dzl_preferences_view_remove_id (DzlPreferences *preferences,
     {
       GtkWidget *widget = tracked->widget;
 
-      /* We have to steal the structure so that we retain access to
-       * the structure after removing it from the hashtable.
-       */
-      g_hash_table_steal (priv->widgets, GUINT_TO_POINTER (widget_id));
+      g_hash_table_remove (priv->widgets, GUINT_TO_POINTER (widget_id));
 
       if (widget != NULL && !gtk_widget_in_destruction (widget))
         {
@@ -883,9 +871,9 @@ dzl_preferences_view_remove_id (DzlPreferences *preferences,
             gtk_widget_destroy (parent);
           else
             gtk_widget_destroy (widget);
-        }
 
-      return TRUE;
+          return TRUE;
+        }
     }
 
   return FALSE;
@@ -955,3 +943,9 @@ dzl_preferences_iface_init (DzlPreferencesInterface *iface)
   iface->remove_id = dzl_preferences_view_remove_id;
   iface->get_widget = dzl_preferences_view_get_widget;
 }
+
+GtkWidget *
+dzl_preferences_view_new (void)
+{
+  return g_object_new (DZL_TYPE_PREFERENCES_VIEW, NULL);
+}
