@@ -19,27 +19,37 @@
 #define G_LOG_DOMAIN "dzl-tree-node"
 
 #include <glib/gi18n.h>
+#include <string.h>
 
 #include "tree/dzl-tree.h"
 #include "tree/dzl-tree-node.h"
 #include "tree/dzl-tree-private.h"
+#include "util/dzl-macros.h"
 
 struct _DzlTreeNode
 {
   GInitiallyUnowned  parent_instance;
 
-  GObject           *item;
   DzlTreeNode       *parent;
+
+  GObject           *item;
   gchar             *text;
   DzlTree           *tree;
-  GQuark             icon_name;
+
   GIcon             *gicon;
   GList             *emblems;
-  guint              use_markup : 1;
-  guint              needs_build : 1;
-  guint              is_dummy : 1;
+  GQuark             icon_name;
+  GQuark             expanded_icon_name;
+
+  GdkRGBA            foreground_rgba;
+
   guint              children_possible : 1;
+  guint              is_dummy : 1;
+  guint              foreground_rgba_set : 1;
+  guint              needs_build_children : 1;
+  guint              reset_on_collapse : 1;
   guint              use_dim_label : 1;
+  guint              use_markup : 1;
 };
 
 typedef struct
@@ -53,10 +63,12 @@ G_DEFINE_TYPE (DzlTreeNode, dzl_tree_node, G_TYPE_INITIALLY_UNOWNED)
 enum {
   PROP_0,
   PROP_CHILDREN_POSSIBLE,
+  PROP_EXPANDED_ICON_NAME,
   PROP_ICON_NAME,
   PROP_GICON,
   PROP_ITEM,
   PROP_PARENT,
+  PROP_RESET_ON_COLLAPSE,
   PROP_TEXT,
   PROP_TREE,
   PROP_USE_DIM_LABEL,
@@ -93,7 +105,14 @@ dzl_tree_node_get_tree (DzlTreeNode *node)
 {
   g_return_val_if_fail (DZL_IS_TREE_NODE (node), NULL);
 
-  return node->tree;
+  while (node != NULL)
+    {
+      if (node->tree != NULL)
+        return node->tree;
+      node = node->parent;
+    }
+
+  return NULL;
 }
 
 /**
@@ -105,25 +124,12 @@ dzl_tree_node_get_tree (DzlTreeNode *node)
  */
 void
 _dzl_tree_node_set_tree (DzlTreeNode *node,
-                        DzlTree     *tree)
+                         DzlTree     *tree)
 {
   g_return_if_fail (DZL_IS_TREE_NODE (node));
   g_return_if_fail (!tree || DZL_IS_TREE (tree));
 
-  if (node->tree != tree)
-    {
-      if (node->tree != NULL)
-        {
-          g_object_remove_weak_pointer (G_OBJECT (node->tree), (gpointer *)&node->tree);
-          node->tree = NULL;
-        }
-
-      if (tree != NULL)
-        {
-          node->tree = tree;
-          g_object_add_weak_pointer (G_OBJECT (node->tree), (gpointer *)&node->tree);
-        }
-    }
+  dzl_set_weak_pointer (&node->tree, tree);
 }
 
 /**
@@ -137,9 +143,9 @@ _dzl_tree_node_set_tree (DzlTreeNode *node,
  */
 void
 dzl_tree_node_insert_sorted (DzlTreeNode            *node,
-                            DzlTreeNode            *child,
-                            DzlTreeNodeCompareFunc  compare_func,
-                            gpointer               user_data)
+                             DzlTreeNode            *child,
+                             DzlTreeNodeCompareFunc  compare_func,
+                             gpointer                user_data)
 {
   g_return_if_fail (DZL_IS_TREE_NODE (node));
   g_return_if_fail (DZL_IS_TREE_NODE (child));
@@ -236,8 +242,8 @@ dzl_tree_node_get_path (DzlTreeNode *node)
 }
 
 gboolean
-dzl_tree_node_get_iter (DzlTreeNode  *self,
-                       GtkTreeIter *iter)
+dzl_tree_node_get_iter (DzlTreeNode *self,
+                        GtkTreeIter *iter)
 {
   gboolean ret = FALSE;
 
@@ -266,6 +272,26 @@ dzl_tree_node_get_parent (DzlTreeNode *node)
   return node->parent;
 }
 
+static void
+dzl_tree_node_queue_draw (DzlTreeNode *self)
+{
+  g_assert (DZL_IS_TREE_NODE (self));
+
+  if (self->tree != NULL)
+    gtk_widget_queue_draw (GTK_WIDGET (self->tree));
+}
+
+void
+dzl_tree_node_set_gicon (DzlTreeNode *self,
+                         GIcon       *gicon)
+{
+  g_return_if_fail (DZL_IS_TREE_NODE (self));
+  g_return_if_fail (!gicon || G_IS_ICON (gicon));
+
+  if (g_set_object (&self->gicon, gicon))
+    g_object_notify_by_pspec (G_OBJECT (self), properties [PROP_GICON]);
+}
+
 /**
  * dzl_tree_node_get_gicon:
  *
@@ -347,6 +373,7 @@ dzl_tree_node_set_icon_name (DzlTreeNode *node,
       g_clear_object (&node->gicon);
       g_object_notify_by_pspec (G_OBJECT (node), properties [PROP_ICON_NAME]);
       g_object_notify_by_pspec (G_OBJECT (node), properties [PROP_GICON]);
+      dzl_tree_node_queue_draw (node);
     }
 }
 
@@ -376,6 +403,7 @@ dzl_tree_node_add_emblem (DzlTreeNode *self,
   self->emblems = g_list_prepend (self->emblems, g_strdup (emblem));
   g_clear_object (&self->gicon);
   g_object_notify_by_pspec (G_OBJECT (self), properties [PROP_GICON]);
+  dzl_tree_node_queue_draw (self);
 }
 
 void
@@ -394,6 +422,7 @@ dzl_tree_node_remove_emblem (DzlTreeNode *self,
           self->emblems = g_list_delete_link (self->emblems, iter);
           g_clear_object (&self->gicon);
           g_object_notify_by_pspec (G_OBJECT (self), properties [PROP_GICON]);
+          dzl_tree_node_queue_draw (self);
           return;
         }
     }
@@ -413,6 +442,7 @@ dzl_tree_node_clear_emblems (DzlTreeNode *self)
   self->emblems = NULL;
   g_clear_object (&self->gicon);
   g_object_notify_by_pspec (G_OBJECT (self), properties [PROP_GICON]);
+  dzl_tree_node_queue_draw (self);
 }
 
 /**
@@ -463,6 +493,7 @@ dzl_tree_node_set_emblems (DzlTreeNode         *self,
 
   g_clear_object (&self->gicon);
   g_object_notify_by_pspec (G_OBJECT (self), properties [PROP_GICON]);
+  dzl_tree_node_queue_draw (self);
 }
 
 /**
@@ -492,20 +523,7 @@ _dzl_tree_node_set_parent (DzlTreeNode *node,
   g_return_if_fail (node->parent == NULL);
   g_return_if_fail (!parent || DZL_IS_TREE_NODE (parent));
 
-  if (parent != node->parent)
-    {
-      if (node->parent != NULL)
-        {
-          g_object_remove_weak_pointer (G_OBJECT (node->parent), (gpointer *)&node->parent);
-          node->parent = NULL;
-        }
-
-      if (parent != NULL)
-        {
-          node->parent = parent;
-          g_object_add_weak_pointer (G_OBJECT (node->parent), (gpointer *)&node->parent);
-        }
-    }
+  dzl_set_weak_pointer (&node->parent, parent);
 }
 
 const gchar *
@@ -579,7 +597,7 @@ dzl_tree_node_get_item (DzlTreeNode *node)
 
 gboolean
 dzl_tree_node_expand (DzlTreeNode *node,
-                     gboolean    expand_ancestors)
+                      gboolean     expand_ancestors)
 {
   DzlTree *tree;
   GtkTreePath *path;
@@ -657,16 +675,19 @@ dzl_tree_node_invalidate (DzlTreeNode *self)
 gboolean
 dzl_tree_node_get_expanded (DzlTreeNode *self)
 {
-  GtkTreePath *path;
   gboolean ret = TRUE;
 
   g_return_val_if_fail (DZL_IS_TREE_NODE (self), FALSE);
 
   if ((self->tree != NULL) && (self->parent != NULL))
     {
-      path = dzl_tree_node_get_path (self);
-      ret = gtk_tree_view_row_expanded (GTK_TREE_VIEW (self->tree), path);
-      gtk_tree_path_free (path);
+      GtkTreePath *path = dzl_tree_node_get_path (self);
+
+      if (path != NULL)
+        {
+          ret = gtk_tree_view_row_expanded (GTK_TREE_VIEW (self->tree), path);
+          gtk_tree_path_free (path);
+        }
     }
 
   return ret;
@@ -680,27 +701,17 @@ dzl_tree_node_finalize (GObject *object)
   g_clear_object (&self->item);
   g_clear_pointer (&self->text, g_free);
 
-  if (self->tree)
-    {
-      g_object_remove_weak_pointer (G_OBJECT (self->tree), (gpointer *)&self->tree);
-      self->tree = NULL;
-    }
-
-  if (self->parent)
-    {
-      g_object_remove_weak_pointer (G_OBJECT (self->parent),
-                                    (gpointer *)&self->parent);
-      self->parent = NULL;
-    }
+  dzl_clear_weak_pointer (&self->tree);
+  dzl_clear_weak_pointer (&self->parent);
 
   G_OBJECT_CLASS (dzl_tree_node_parent_class)->finalize (object);
 }
 
 static void
 dzl_tree_node_get_property (GObject    *object,
-                           guint       prop_id,
-                           GValue     *value,
-                           GParamSpec *pspec)
+                            guint       prop_id,
+                            GValue     *value,
+                            GParamSpec *pspec)
 {
   DzlTreeNode *node = DZL_TREE_NODE (object);
 
@@ -710,6 +721,10 @@ dzl_tree_node_get_property (GObject    *object,
       g_value_set_boolean (value, dzl_tree_node_get_children_possible (node));
       break;
 
+    case PROP_EXPANDED_ICON_NAME:
+      g_value_set_string (value, _dzl_tree_node_get_expanded_icon (node));
+      break;
+
     case PROP_ICON_NAME:
       g_value_set_string (value, g_quark_to_string (node->icon_name));
       break;
@@ -726,6 +741,10 @@ dzl_tree_node_get_property (GObject    *object,
       g_value_set_object (value, node->parent);
       break;
 
+    case PROP_RESET_ON_COLLAPSE:
+      g_value_set_boolean (value, node->reset_on_collapse);
+      break;
+
     case PROP_TEXT:
       g_value_set_string (value, node->text);
       break;
@@ -749,9 +768,9 @@ dzl_tree_node_get_property (GObject    *object,
 
 static void
 dzl_tree_node_set_property (GObject      *object,
-                           guint         prop_id,
-                           const GValue *value,
-                           GParamSpec   *pspec)
+                            guint         prop_id,
+                            const GValue *value,
+                            GParamSpec   *pspec)
 {
   DzlTreeNode *node = DZL_TREE_NODE (object);
 
@@ -761,6 +780,14 @@ dzl_tree_node_set_property (GObject      *object,
       dzl_tree_node_set_children_possible (node, g_value_get_boolean (value));
       break;
 
+    case PROP_EXPANDED_ICON_NAME:
+      node->expanded_icon_name = g_quark_from_string (g_value_get_string (value));
+      break;
+
+    case PROP_GICON:
+      dzl_tree_node_set_gicon (node, g_value_get_object (value));
+      break;
+
     case PROP_ICON_NAME:
       dzl_tree_node_set_icon_name (node, g_value_get_string (value));
       break;
@@ -769,6 +796,10 @@ dzl_tree_node_set_property (GObject      *object,
       dzl_tree_node_set_item (node, g_value_get_object (value));
       break;
 
+    case PROP_RESET_ON_COLLAPSE:
+      dzl_tree_node_set_reset_on_collapse (node, g_value_get_boolean (value));
+      break;
+
     case PROP_TEXT:
       dzl_tree_node_set_text (node, g_value_get_string (value));
       break;
@@ -815,7 +846,12 @@ dzl_tree_node_class_init (DzlTreeNodeClass *klass)
                           FALSE,
                           (G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
-
+  properties [PROP_EXPANDED_ICON_NAME] =
+    g_param_spec_string ("expanded-icon-name",
+                         "Expanded Icon Name",
+                         "The icon-name to use when the row is expanded",
+                         NULL,
+                         G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
 
   /**
    * DzlTreeNode:icon-name:
@@ -827,7 +863,7 @@ dzl_tree_node_class_init (DzlTreeNodeClass *klass)
                          "Icon Name",
                          "The icon name to display.",
                          NULL,
-                         G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
+                         G_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY | G_PARAM_STATIC_STRINGS);
   /**
    * DzlTreeNode:gicon:
    *
@@ -838,7 +874,7 @@ dzl_tree_node_class_init (DzlTreeNodeClass *klass)
                          "GIcon",
                          "The GIcon object",
                          G_TYPE_ICON,
-                         G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
+                         G_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY | G_PARAM_STATIC_STRINGS);
 
   /**
    * DzlTreeNode:item:
@@ -850,7 +886,7 @@ dzl_tree_node_class_init (DzlTreeNodeClass *klass)
                          "Item",
                          "Optional object to associate with node.",
                          G_TYPE_OBJECT,
-                         G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
+                         G_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY | G_PARAM_STATIC_STRINGS);
 
   /**
    * DzlTreeNode:parent:
@@ -864,6 +900,24 @@ dzl_tree_node_class_init (DzlTreeNodeClass *klass)
                          DZL_TYPE_TREE_NODE,
                          G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
 
+  /**
+   * DzlTreeNode:reset-on-collapse:
+   *
+   * The "reset-on-collapse" property denotes that all children should be
+   * removed from the node when it's row is collapsed. It will also set
+   * #DzlTreeNode:needs-build to %TRUE so the next expansion rebuilds the
+   * children. This is useful for situations where you want to ensure the nodes
+   * are up to date (refreshed) on every expansion.
+   *
+   * Since: 3.28
+   */
+  properties [PROP_RESET_ON_COLLAPSE] =
+    g_param_spec_boolean ("reset-on-collapse",
+                          "Reset on Collapse",
+                          "Reset by clearing children on collapse, requiring a rebuild on next expand",
+                          FALSE,
+                          G_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY | G_PARAM_STATIC_STRINGS);
+
   /**
    * DzlTreeNode:tree:
    *
@@ -886,7 +940,7 @@ dzl_tree_node_class_init (DzlTreeNodeClass *klass)
                          "Text",
                          "The text of the node.",
                          NULL,
-                         G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
+                         G_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY | G_PARAM_STATIC_STRINGS);
 
   /**
    * DzlTreeNode:use-markup:
@@ -898,14 +952,14 @@ dzl_tree_node_class_init (DzlTreeNodeClass *klass)
                           "Use Markup",
                           "If text should be translated as markup.",
                           FALSE,
-                          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
+                          G_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY | G_PARAM_STATIC_STRINGS);
 
   properties [PROP_USE_DIM_LABEL] =
     g_param_spec_boolean ("use-dim-label",
                           "Use Dim Label",
                           "If text should be rendered with a dim label.",
                           FALSE,
-                          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
+                          G_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY | G_PARAM_STATIC_STRINGS);
 
   g_object_class_install_properties (object_class, LAST_PROP, properties);
 }
@@ -913,7 +967,7 @@ dzl_tree_node_class_init (DzlTreeNodeClass *klass)
 static void
 dzl_tree_node_init (DzlTreeNode *node)
 {
-  node->needs_build = TRUE;
+  node->needs_build_children = TRUE;
 }
 
 static gboolean
@@ -1028,23 +1082,31 @@ dzl_tree_node_show_popover (DzlTreeNode *self,
     }
 }
 
+gboolean 
+_dzl_tree_node_is_dummy (DzlTreeNode *self)
+{
+  g_return_val_if_fail (DZL_IS_TREE_NODE (self), FALSE);
+
+  return self->is_dummy;
+}
+
 gboolean
-_dzl_tree_node_get_needs_build (DzlTreeNode *self)
+_dzl_tree_node_get_needs_build_children (DzlTreeNode *self)
 {
   g_assert (DZL_IS_TREE_NODE (self));
 
-  return self->needs_build;
+  return self->needs_build_children;
 }
 
 void
-_dzl_tree_node_set_needs_build (DzlTreeNode *self,
-                               gboolean    needs_build)
+_dzl_tree_node_set_needs_build_children (DzlTreeNode *self,
+                                         gboolean     needs_build_children)
 {
   g_assert (DZL_IS_TREE_NODE (self));
 
-  self->needs_build = !!needs_build;
+  self->needs_build_children = !!needs_build_children;
 
-  if (!needs_build)
+  if (!needs_build_children)
     self->is_dummy = FALSE;
 }
 
@@ -1061,10 +1123,13 @@ _dzl_tree_node_add_dummy_child (DzlTreeNode *self)
   model = _dzl_tree_get_store (self->tree);
   dzl_tree_node_get_iter (self, &parent);
   dummy = g_object_ref_sink (dzl_tree_node_new ());
+  _dzl_tree_node_set_tree (dummy, self->tree);
+  _dzl_tree_node_set_parent (dummy, self);
+  dummy->is_dummy = TRUE;
   gtk_tree_store_insert_with_values (model, &iter, &parent, -1,
                                      0, dummy,
                                      -1);
-  g_object_unref (dummy);
+  g_clear_object (&dummy);
 }
 
 void
@@ -1084,7 +1149,8 @@ _dzl_tree_node_remove_dummy_child (DzlTreeNode *self)
   if (dzl_tree_node_get_iter (self, &iter) &&
       gtk_tree_model_iter_children (GTK_TREE_MODEL (model), &children, &iter))
     {
-      while (gtk_tree_store_remove (model, &children)) { }
+      while (gtk_tree_store_remove (model, &children))
+        { /* Do nothing */ }
     }
 }
 
@@ -1107,7 +1173,7 @@ dzl_tree_node_get_children_possible (DzlTreeNode *self)
  */
 void
 dzl_tree_node_set_children_possible (DzlTreeNode *self,
-                                    gboolean    children_possible)
+                                     gboolean     children_possible)
 {
   g_return_if_fail (DZL_IS_TREE_NODE (self));
 
@@ -1117,7 +1183,7 @@ dzl_tree_node_set_children_possible (DzlTreeNode *self,
     {
       self->children_possible = children_possible;
 
-      if (self->tree && self->needs_build)
+      if (self->tree != NULL && self->needs_build_children)
         {
           if (self->children_possible)
             _dzl_tree_node_add_dummy_child (self);
@@ -1157,3 +1223,144 @@ dzl_tree_node_is_root (DzlTreeNode *node)
 
   return node->parent == NULL;
 }
+
+const gchar *
+_dzl_tree_node_get_expanded_icon (DzlTreeNode *node)
+{
+  g_return_val_if_fail (DZL_IS_TREE_NODE (node), NULL);
+
+  return g_quark_to_string (node->expanded_icon_name);
+}
+
+gboolean
+dzl_tree_node_get_reset_on_collapse (DzlTreeNode *self)
+{
+  g_return_val_if_fail (DZL_IS_TREE_NODE (self), FALSE);
+
+  return self->reset_on_collapse;
+}
+
+void
+dzl_tree_node_set_reset_on_collapse (DzlTreeNode *self,
+                                     gboolean     reset_on_collapse)
+{
+  g_return_if_fail (DZL_IS_TREE_NODE (self));
+
+  reset_on_collapse = !!reset_on_collapse;
+
+  if (reset_on_collapse != self->reset_on_collapse)
+    {
+      self->reset_on_collapse = reset_on_collapse;
+      g_object_notify_by_pspec (G_OBJECT (self), properties [PROP_RESET_ON_COLLAPSE]);
+    }
+}
+
+guint
+dzl_tree_node_n_children (DzlTreeNode *self)
+{
+  g_return_val_if_fail (DZL_IS_TREE_NODE (self), 0);
+
+  if (!self->needs_build_children && self->tree != NULL)
+    {
+      GtkTreeIter iter;
+      GtkTreeModel *model;
+
+      model = GTK_TREE_MODEL (_dzl_tree_get_store (self->tree));
+
+      if (dzl_tree_node_get_iter (self, &iter))
+        return gtk_tree_model_iter_n_children (model, &iter);
+    }
+
+  return 0;
+}
+
+/**
+ * dzl_tree_node_nth_child:
+ * @self: a #DzlTreeNode
+ * @nth: the index of the child
+ *
+ * Gets the @nth child of @self or %NULL if it does not exist.
+ *
+ * Returns: (transfer full) (nullable): a #DzlTreeNode or %NULL
+ */
+DzlTreeNode *
+dzl_tree_node_nth_child (DzlTreeNode *self,
+                         guint        nth)
+{
+  g_return_val_if_fail (DZL_IS_TREE_NODE (self), NULL);
+  g_return_val_if_fail (!self->needs_build_children, NULL);
+
+  if (self->tree != NULL)
+    {
+      GtkTreeIter parent;
+      GtkTreeIter iter;
+      GtkTreeModel *model;
+
+      model = GTK_TREE_MODEL (_dzl_tree_get_store (self->tree));
+
+      if (dzl_tree_node_get_iter (self, &parent) &&
+          gtk_tree_model_iter_nth_child (model, &iter, &parent, nth))
+        {
+          g_autoptr(DzlTreeNode) node = NULL;
+
+          gtk_tree_model_get (model, &iter, 0, &node, -1);
+          g_assert (DZL_IS_TREE_NODE (node));
+
+          /* Don't hand back a dummy node */
+          if (_dzl_tree_node_is_dummy (node))
+            return NULL;
+
+          return g_steal_pointer (&node);
+        }
+    }
+
+  return NULL;
+}
+
+/**
+ * dzl_tree_node_get_foreground_rgba:
+ * @self: a #DzlTreeNode
+ *
+ * Gets the foreground-rgba to use for row text.
+ *
+ * If %NULL, the default foreground color should be used.
+ *
+ * Returns: (nullable) (transfer none): A #GdkRGBA or %NULL
+ *
+ * Since: 3.28
+ */
+const GdkRGBA *
+dzl_tree_node_get_foreground_rgba (DzlTreeNode *self)
+{
+  g_return_val_if_fail (DZL_IS_TREE_NODE (self), NULL);
+
+  if (self->foreground_rgba_set)
+    return &self->foreground_rgba;
+
+  return NULL;
+}
+
+/**
+ * dzl_tree_node_set_foreground_rgba:
+ * @self: a #DzlTreeNode
+ * @foreground_rgba: (nullable): A #GdkRGBA or %NULL
+ *
+ * Sets the foreground-rgba to be used by the row text.
+ *
+ * If @foreground_rgba is %NULL, the value is reset to the default.
+ *
+ * Since: 3.28
+ */
+void
+dzl_tree_node_set_foreground_rgba (DzlTreeNode   *self,
+                                   const GdkRGBA *foreground_rgba)
+{
+  g_return_if_fail (DZL_IS_TREE_NODE (self));
+
+  if (foreground_rgba != NULL)
+    self->foreground_rgba = *foreground_rgba;
+  else
+    memset (&self->foreground_rgba, 0, sizeof self->foreground_rgba);
+
+  self->foreground_rgba_set = !!foreground_rgba;
+}
